//  Leonard Bencloski (lmb92@pitt.edu)
//  Assignment #2
//  CS1520
//
//  Black Jack

// Black jack server.  Listens for new client connections, starts a new hand, shuffles a deck
// and checks results of a hand.

import java.net.*;
import java.io.*;
import java.util.*;

public class Server {

  public static final int PORT = 5678;

  public static void main( String [] argv ) throws IOException
  {
	System.out.println("Black Jack Server has started and is listening on port: " + PORT);
    ServerSocket ss = new ServerSocket(PORT);
    System.out.println(ss);

    while ( true )
      new ServerConnection( ss.accept() ).start(  );
  }

} // end of class Server

class ServerConnection extends Thread
{
	boolean loop = true;
	boolean t = true;
	boolean ok = true;
	Deck d;
	Card temp;
	Card temp2;
	short scoreDealer=0;
	short scoreClient=0;
	String message="";
	ArrayList serCards = new ArrayList();
	ArrayList serCards2 = new ArrayList();
	ArrayList cliCards = new ArrayList();
	short countAce = 0;
	short dealerAce = 0;
	short send = 0;

    Socket client;

  ServerConnection ( Socket client ) throws SocketException, IOException
  {
    this.client = client;
    setPriority( NORM_PRIORITY - 1 );
    System.out.println("");
    System.out.println("A new client has connected.");
  }

  public void run(  )
  {
	try
	{
		 ObjectInputStream in = new ObjectInputStream( client.getInputStream(  ) );
		 ObjectOutputStream out = new ObjectOutputStream( client.getOutputStream(  ) );

		d = new Deck();  // Creates a new deck and shuffles it.
		d.shuffle();

     while ( ok )
      {
			Object obj = in.readObject();  //Reads in an object from the client
			game(obj, out); // Sends it to game to check what to do.

      }

   }
	// If there are any problems, disconnect.
	catch ( SocketException e6 )
	{
		 removeIt();
		 ok = false;
	}
	catch ( EOFException e2 )
	{

		removeIt();
		ok = false;
	}
	catch ( IOException e )//3
	{

		removeIt();
		ok = false;

	}

	catch ( ClassNotFoundException e3 )
	{
	 	removeIt();
		ok = false;
	}


}

  // Removes a client thread.
  private synchronized void removeIt()
  {
	    try
        {
            client.close();
            System.out.println( "A client has disconnected.");
        }
        catch (IOException e)
        {
            System.out.println("Client already left.");
        }
  }

  private synchronized void game(Object x, ObjectOutputStream out )
  {
	  try
	  {		// If a newCard, user has requested a hit, send a new card to them.
			if( x instanceof newCard )
			{
			  temp = d.getCard();
		      out.writeObject( temp );
		      out.flush(  );
		      if(temp.isAce)
		      	countAce++;  //Checks and counts the number of aces.

		      cliCards.add(temp);
		      dealerResult deal = new dealerResult();
		      scoreClient += temp.numValue; // Adds up a players score.

			   if(scoreClient<21) // Checks results of a hit
			   {
				  deal.x = 3;
				  deal.score = scoreClient;
			   }

			  else if(scoreClient>21)
			  {
				  while( (countAce > 0) && (scoreClient > 21) ) //If over 21 and have Aces, adjust score
				  {
						countAce--;
						scoreClient -= 10;
				  }

				if(scoreClient>21)
				{
					deal.x = 1; //Dealer wins
					deal.score = scoreClient;
				}
				else if(scoreClient==21)
				{
					deal.x = 0; // Player wins
					deal.score = scoreClient;
				}
				else if(scoreClient<21)
				{
					deal.x = 3; // Keep playing
					deal.score = scoreClient;
				}
			 }
		      if(scoreClient==21)
		      {
				  deal.x = 0; // Player wins
				  deal.score = scoreClient;
			  }


			out.writeObject( deal );
		    out.flush(  );
			}//IF

			// Sends cards for a new hand, checks for blackjack.
			else if(x instanceof newHand)
			{
				serCards.clear();
				serCards2.clear();
				cliCards.clear();
				countAce=0;
				dealerAce=0;
				send = 0;
				loop=true;
				dealerResult deal = new dealerResult();

				if(d.cardCount<20)
				{
					d.cardCount=51;
					d.shuffle();
				}

				scoreDealer=0;
				scoreClient=0;
				message="";


				out.writeObject( new dealerCard() ); //Tells client that cards are dealers
				out.flush();
				for(int g=0; g<2; g++)
				{
					temp = d.getCard();
					scoreDealer += temp.numValue;
					out.writeObject( temp );
					out.flush();
					if(temp.isAce)
		      			dealerAce++;
					serCards.add(temp);
				}

				out.writeObject( new clientCard () ); //Tells client that cards are players'.
				out.flush();

				for(int h=0; h<2; h++)
				{
					temp2 = d.getCard();
					scoreClient += temp2.numValue;
					out.writeObject( temp2 );
					out.flush();
					if(temp2.isAce)
						countAce++;

					cliCards.add(temp2);
				}

				if(scoreClient == 21) //Checks for blackjack
				{
					deal.x = 0;
					deal.score = scoreClient;
					deal.dealer = scoreDealer;
				}

				if(scoreDealer == 21)
				{
					deal.x = 1;
					deal.score = scoreClient;
					deal.dealer = scoreDealer;
				}

				if(scoreDealer == 21 && scoreClient == 21)
				{
					deal.x = 2;
					deal.score = scoreClient;
					deal.dealer = scoreDealer;
				}

				if(scoreDealer != 21 && scoreClient != 21)
				{
					deal.x = 3; //Keep playing
					deal.score = scoreClient;
					deal.dealer = scoreDealer;
				}
				if(scoreDealer == 22) //If there are 2 Aces, adjust score
				{
					deal.x = 3;
					scoreDealer-=10;
					dealerAce--;
					deal.score = scoreClient;
					deal.dealer = scoreDealer;
				}
				if(scoreClient == 22)
				{
					deal.x = 3;
					scoreClient-=10;
					countAce--;
					deal.score = scoreClient;
					deal.dealer = scoreDealer;
				}

				out.writeObject( deal );

			}

			// Closes a user thread.
			else if( x instanceof Close )
			{
				ok = false;
				removeIt();
			}

			// If a player stands, send back dealer cards and result of hand.
			else if( x instanceof playerStand )
			{
				send = 0;
				playerStand stand = new playerStand();

				while(loop)
				{
					if(scoreDealer < 17) // If dealer < 17, get another card.
					{

						temp = d.getCard();
						scoreDealer += temp.numValue;
						if(temp.isAce)
							dealerAce++;

						serCards2.add(temp);
						send++;
						stand.dealer = scoreDealer;
					}

					if(scoreDealer > 21) // If dealer > 21, check for Aces and adjust score
					{
						while( (dealerAce > 0) && (scoreDealer > 21) )
						{
							dealerAce--;
							scoreDealer -= 10;
						}

					}

					if(scoreDealer >= 17)  // Stand if >= 21
					{
						stand.dealer = scoreDealer;
						loop = false;
					}

				}//while

				// Check results and send to client
				if(scoreClient > scoreDealer && scoreClient <= 21)
					stand.x = 0; // player wins
				if(scoreClient > scoreDealer && scoreClient > 21)
					stand.x = 1; //dealer wins
				if(scoreClient < scoreDealer && scoreDealer > 21)
					stand.x = 0; //player wins
				if(scoreClient < scoreDealer && scoreDealer <= 21)
					stand.x = 1; //dealer
				if(scoreClient == scoreDealer)
					stand.x = 2; //push, tie

				stand.count = (short) serCards2.size(); //Sets how many cards should be sent back
				stand.dealer = scoreDealer;
				out.writeObject( stand );
				out.flush();

				send sender = new send(); // If there are cards to be sent back, set true.
				sender.ok = true;

				if(serCards2.size() > 0) // Send back the cards
				{
					out.writeObject(  sender );
					out.flush();
					for(int z=0; z<serCards2.size(); z++)
					{
							Card sd = (Card) serCards2.get(z);
							out.writeObject( sd );
							out.flush();
					}
				}
				else
				{
					sender.ok = false;
					out.writeObject(  sender );
					out.flush();
				}

			}

	  }//Try

	// Disconnect if any error occurs.
	catch ( SocketException e6 )
	{
		removeIt();
		ok = false;
	}
	catch ( EOFException e2 )
	{
		removeIt();
		ok = false;
	}
	catch ( IOException e )//3
	{
		removeIt();
		ok = false;
	}


  }


}//End
